include EuSDL2.e
--include myLibs/common.e
--include myLibs/myDebug.e
include button.e

public constant ID=1, CLASS=2, X1=3, Y1=4, X2=5, Y2=6, RTN_CLICK=7, RTN_OVER=8, PARAMS=9
--constant EVT_CLICK=1, EVT_OVER=2

integer old_gmouse = 0, old_mouseX = 0, old_mouseY = 0

-- stores widgets
-- widgets are visible components of the GUI
public sequence registry

registry = {}

function new_id()
  return length(registry) + 1
end function

function find_widget(integer id)
   for i = 1 to length(registry) do
    if compare(registry[i][ID], id) = 0 then
      return i
    end if
  end for
  return 0
end function

function get_class(integer id)
  integer found = find_widget(id)
  if not found then return -1 end if
  return registry[found][CLASS]
end function

-- widget registered after creation
function registrate_widget(sequence class, integer x1, integer y1, integer x2, integer y2, integer rtn_click, integer rtn_over, sequence params)
  integer id = new_id()
  registry = append(registry, {id, class, x1, y1, x2, y2, rtn_click, rtn_over, params})
  return id
end function

public function create_widget(sequence class, integer x1, integer y1, integer x2, integer y2, integer rtn_click=0, integer rtn_over=0, sequence params={})
  sequence text, image
  integer state

  if find(class, {"Button", "ToggleButton"}) then   -- text button
    text = params[1]
    if equal(class, "Button") then
      state = 0
    elsif equal(class, "ToggleButton") then
      state = params[2]
    end if
  elsif find(class, {"PictureButton", "TogglePictureButton"}) then   -- picture button
    image = params[1]
    if equal(class, "PictureButton") then
      state = 0
    elsif equal(class, "TogglePictureButton") then
      state = params[2]
    end if
  end if
  return registrate_widget(class, x1, y1, x2, y2, rtn_click, rtn_over, params)
end function

public procedure set_on_click_routine(integer id, integer Routine)
  integer found = find_widget(id)
  if not found then return end if
  registry[found][RTN_CLICK] = Routine
end procedure

public procedure set_on_over_routine(integer id, integer Routine)
  integer found = find_widget(id)
  if not found then return end if
  registry[found][RTN_OVER] = Routine
end procedure

public procedure test_widget_hit(integer gmouse, integer mouseX, integer mouseY)
  -- on click
  if gmouse != old_gmouse then
    printf(1, "gmouse = %d\n", gmouse)
    for i = 1 to length(registry) do
      sequence vc = registry[i]
      if (mouseX >= vc[X1]) and (mouseX <= vc[X2]) and
         (mouseY >= vc[Y1]) and (mouseY <= vc[Y2]) then
        if equal(vc[CLASS], "Button") then
          vc[PARAMS][2] =  (gmouse = SDL_BUTTON_LEFT)
        elsif equal(vc[CLASS], "ToggleButton") and (gmouse = SDL_BUTTON_LEFT) then
          vc[PARAMS][2] =  (1 - vc[PARAMS][2])
        end if
        registry[i] = vc
        -- call widget procedure
        if (gmouse = SDL_BUTTON_LEFT) and (vc[RTN_CLICK] != 0) then call_proc(vc[RTN_CLICK], {i}) end if
      end if
    end for
    old_gmouse = gmouse
  end if
  -- on over
  if (mouseX != old_mouseX) or (mouseY != old_mouseY) then
    for i = 1 to length(registry) do
      sequence vc = registry[i]
      if (mouseX >= vc[X1]) and (mouseX <= vc[X2]) and
         (mouseY >= vc[Y1]) and (mouseY <= vc[Y2]) then
        -- call widget procedure
        if vc[RTN_OVER] != 0 then call_proc(vc[RTN_OVER], {i}) end if
      end if
    end for
    old_mouseX = mouseX
    old_mouseY = mouseY
  end if
end procedure

public procedure draw_widgets(atom ren)
  for i = 1 to length(registry) do
    sequence vc = registry[i]
    if find(vc[CLASS], {"Button", "ToggleButton"}) then
      text_button(ren, vc[X1], vc[Y1], vc[X2], vc[Y2], vc[PARAMS][1], vc[PARAMS][2])
    end if
  end for
end procedure

